@@ -31,6 +31,7 @@ typedef enum {
     RESULT_TYPE_VERIFY,
     RESULT_TYPE_DECRYPT,
     RESULT_TYPE_SIGN,
+    RESULT_TYPE_ENCRYPT
 } ResultType;
 
 
@@ -72,9 +73,11 @@ struct gpgme_context_s {
         VerifyResult verify;
         DecryptResult decrypt;
         SignResult sign;
+        EncryptResult encrypt;
     } result;
 
     GpgmeData notation;    /* last signature notation */
+    GpgmeData op_info;     /* last operation info */
 
     GpgmeKey tmp_key;       /* used by keylist.c */
     volatile int key_cond;  /* something new is available */
