@@ -95,10 +95,14 @@ _gpgme_release_result ( GpgmeCtx c )
       case RESULT_TYPE_SIGN:
         _gpgme_release_sign_result ( c->result.sign );
         break;
+      case RESULT_TYPE_ENCRYPT:
+        _gpgme_release_encrypt_result ( c->result.encrypt );
+        break;
     }
 
     c->result.verify = NULL;
     c->result_type = RESULT_TYPE_NONE;
+    _gpgme_set_op_info (c, NULL);
 }
 
 
@@ -123,9 +127,10 @@ gpgme_cancel (GpgmeCtx c)
  * gpgme_get_notation:
  * @c: the context 
  * 
- * If there is notation data available from the last signature check, this
- * function may be used to return this notation data as a string.  The string
- * is an XML represantaton of that data embedded in a %<notation> container.
+ * If there is notation data available from the last signature check,
+ * this function may be used to return this notation data as a string.
+ * The string is an XML represantaton of that data embedded in a
+ * %&lt;notation&gt; container.
  * 
  * Return value: An XML string or NULL if no notation data is available.
  **/
@@ -137,6 +142,60 @@ gpgme_get_notation ( GpgmeCtx c )
     return _gpgme_data_get_as_string ( c->notation );
 }
 
+
+/**
+ * gpgme_get_op_info:
+ * @c: the context 
+ * @reserved: 
+ * 
+ * Return information about the last information.  The caller has to
+ * free the string.  NULL is returned if there is not previous
+ * operation available or the operation has not yet finished.
+ *
+ * Here is a sample information we return:
+ * <literal>
+ * <![CDATA[
+ * <GnupgOperationInfo>
+ *   <signature>
+ *     <detached/> <!-- or cleartext or standard -->
+ *     <algo>17</algo>
+ *     <hashalgo>2</hashalgo>
+ *     <micalg>pgp-sha1</micalg>
+ *     <sigclass>01</sigclass>
+ *     <created>9222222</created>
+ *     <fpr>121212121212121212</fpr>
+ *   </signature>
+ * </GnupgOperationInfo>
+ * ]]>
+ * </literal>
+ * Return value: NULL for no info available or an XML string 
+ **/
+char *
+gpgme_get_op_info ( GpgmeCtx c, int reserved )
+{
+    if (!c || reserved)
+        return NULL; /*invalid value */
+ 
+    return _gpgme_data_get_as_string (c->op_info);
+}
+
+/*
+ * Store the data object with the operation info in the
+ * context. Caller should not use that object anymore.  
+ */
+void
+_gpgme_set_op_info (GpgmeCtx c, GpgmeData info)
+{
+    assert (c);
+
+    gpgme_data_release (c->op_info); 
+    c->op_info = NULL;
+
+    if (info)
+        c->op_info = info;
+}
+
+
 /**
  * gpgme_set_armor:
  * @c: the contect 
@@ -152,6 +211,23 @@ gpgme_set_armor ( GpgmeCtx c, int yes )
     c->use_armor = yes;
 }
 
+
+/**
+ * gpgme_get_armor:
+ * @c: the context
+ * 
+ * Return the state of the armor flag which can be changed using
+ * gpgme_set_armor().
+ * 
+ * Return value: Boolean whether armor mode is to be used.
+ **/
+int 
+gpgme_get_armor (GpgmeCtx c)
+{
+    return c && c->use_armor;
+}
+
+
 /**
  * gpgme_set_textmode:
  * @c: the context 
@@ -168,6 +244,23 @@ gpgme_set_textmode ( GpgmeCtx c, int yes )
     c->use_textmode = yes;
 }
 
+/**
+ * gpgme_get_textmode:
+ * @c: the context
+ * 
+ * Return the state of the textmode flag which can be changed using
+ * gpgme_set_textmode().
+ * 
+ * Return value: Boolean whether textmode is to be used.
+ **/
+int 
+gpgme_get_textmode (GpgmeCtx c)
+{
+    return c && c->use_textmode;
+}
+
+
+
 /**
  * gpgme_set_keylist_mode:
  * @c: the context
