@@ -89,10 +89,10 @@ set_userid_flags ( GpgmeKey key, const char *s )
           case 'r': key->uids->revoked  = 1; break;
           case 'i': key->uids->invalid  = 1; break;
 
-          case 'n': key->uids->validity = 1; break;
-          case 'm': key->uids->validity = 2; break;
-          case 'f': key->uids->validity = 3; break;
-          case 'u': key->uids->validity = 4; break;
+          case 'n': key->uids->validity = GPGME_VALIDITY_NEVER; break;
+          case 'm': key->uids->validity = GPGME_VALIDITY_MARGINAL; break;
+          case 'f': key->uids->validity = GPGME_VALIDITY_FULL; break;
+          case 'u': key->uids->validity = GPGME_VALIDITY_ULTIMATE; break;
         }
     }
 }
@@ -156,8 +156,11 @@ keylist_colon_handler ( GpgmeCtx ctx, char *line )
 
     if ( ctx->out_of_core )
         return;
-    if (!line)
-        return; /* EOF */
+    if (!line) { /* EOF */
+        if (ctx->tmp_key)
+            finish_key (ctx);
+        return; 
+    }
 
     for (p = line; p; p = pend) {
         field++;
@@ -373,10 +376,12 @@ finish_key ( GpgmeCtx ctx )
 /**
  * gpgme_op_keylist_start:
  * @c: context 
- * @pattern: a GnuPg user ID or NULL for all
+ * @pattern: a GnuPG user ID or NULL for all
  * @secret_only: List only keys where the secret part is available
  * 
- * Note that this function also cancels a pending key listing operaton..
+ * Note that this function also cancels a pending key listing
+ * operaton. To actually retrieve the key, use
+ * gpgme_op_keylist_next().
  * 
  * Return value:  0 on success or an errorcode. 
  **/
@@ -440,6 +445,17 @@ gpgme_op_keylist_start ( GpgmeCtx c,  const char *pattern, int secret_only )
 }
 
 
+/**
+ * gpgme_op_keylist_next:
+ * @c: Context
+ * @r_key: Returned key object
+ * 
+ * Return the next key from the key listing started with
+ * gpgme_op_keylist_start().  The caller must free the key using 
+ * gpgme_key_release().
+ * 
+ * Return value: 0 on success, %GPGME_EOF or anoter error code.
+ **/
 GpgmeError
 gpgme_op_keylist_next ( GpgmeCtx c, GpgmeKey *r_key )
 {
