@@ -26,6 +26,7 @@
 
 /*-- gpgme.c --*/
 void _gpgme_release_result ( GpgmeCtx c );
+void _gpgme_set_op_info (GpgmeCtx c, GpgmeData info);
 
 /*-- wait.c --*/
 GpgmeCtx _gpgme_wait_on_condition ( GpgmeCtx c,
@@ -75,6 +76,9 @@ void _gpgme_release_decrypt_result ( DecryptResult res );
 /*-- sign.c --*/
 void _gpgme_release_sign_result ( SignResult res );
 
+/*-- encrypt.c --*/
+void _gpgme_release_encrypt_result ( EncryptResult res );
+
 
 #endif /* OPS_H */
 
