@@ -34,9 +34,10 @@
 const char *
 _gpgme_get_gpg_path (void)
 {
-    /*   #warning Forced to development version
-    return "/home/wk/work/gnupg-stable/g10/gpg";
-    */
+    /* #warning Forced to take GPG development version
+     *  return "/home/wk/work/gnupg-stable/g10/gpg";
+     */
+
     return GPG_PATH;
 }
 
